package org.ninjasoft.cryptoslam.ui;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.ninjasoft.cryptoslam.data.Cryptogram;

public class CryptoslamApplet extends Applet {
    public static final String COPYRIGHT = 
        "Cryptoslam Applet / Copyright 2005, Brian Enigma <brian@netninja.com>";
    private Cryptogram cryptogram = new Cryptogram();
    //private JTextArea messageText = new JTextArea(20,30);
    //private JTextPane messageText = new JTextPane(new javax.swing.text.html.HTMLDocument());
    private JEditorPane messageText = new JEditorPane("text/html", "");
    private JTextArea keyText = new JTextArea(2, 30);
    private JTextArea menuText = new JTextArea(2, 30);
    private KeyHandler keyHandler = new KeyHandler();
    private int mode = 0;
    private char key1, key2;
    public void init()  {
        initializeGraphics();
        initializeEvents();
    }
    
    public void start() {
        mode = 0;
        refresh();
    }
    
    private void initializeGraphics() {
        Font mono = new Font("Monospaced", Font.PLAIN, 12);
        Font sans = new Font("SansSerif", Font.PLAIN, 10);
        this.setLayout(new BorderLayout());
        messageText.setFont(mono);
        keyText.setFont(mono);
        menuText.setFont(sans);
        this.add(new JScrollPane(messageText, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.CENTER);
        this.add(keyText, BorderLayout.SOUTH);
        this.add(menuText, BorderLayout.NORTH);
        messageText.setEditable(false);
        keyText.setEditable(false);
        //messageText.setForeground(Color.BLACK);
        messageText.setBackground(Color.WHITE);
        keyText.setForeground(Color.BLACK);
        keyText.setBackground(Color.WHITE);
        menuText.setForeground(new Color(0x99, 0x00, 0x00));
        menuText.setBackground(new Color(0xff, 0xff, 0xee));
    }
    
    private void initializeEvents() {
        this.addKeyListener(keyHandler);
        messageText.addKeyListener(keyHandler);
        keyText.addKeyListener(keyHandler);
        menuText.addKeyListener(keyHandler);
    }

    private void refresh() {
        messageText.setText(cryptogram.getFormattedMessage(true));
        keyText.setText(cryptogram.getFormattedAlphabet());
        updateMenu();
    }
    
    private String pad(String s, int length) {
        while (s.length() < length)
            s = s + " ";
        return s;
    }
    
    public Color getBackground() {
        return Color.WHITE;
    }
    
    private void updateMenu() {
        switch(mode) {
        case 1:
            menuText.setText("Ciphertext letter to set? ");
            break;
        case 2:
            menuText.setText("Set letter \"" + key1 + "\" to which plaintext lettre (spacebar to unset)?");
            break;
        case 3:
            menuText.setText("Ciphertext letter to unset? ");
            break;
        case 4:
            menuText.setText("Reset all of the letters?  Are you sure? (y/n) ");
            break;
        case 5:
            menuText.setText("[E]nter message to solve / [Q]uit to main menu");
            break;
        case 6:
            menuText.setText("[L]etter distribution / [W]ord distribution / [R]andom cipher / RO[T] / [Q]uit to main menu");
            break;
        case 0:
        default:
            mode = 0;
            menuText.setText(COPYRIGHT + "\nMenu: [S]et letter / [U]nset letter / [R]eset / [C]opy to clipboard / [M]essage / [T]ools");
            break;
        }
    }
    
    public class KeyHandler extends KeyAdapter {
        public void keyTyped(KeyEvent e) {
            StringBuffer tempStr = new StringBuffer();
            char letters[] = new char[26];
            int counts[] = new int[26];
            switch (mode) {
            case 0:
                switch(e.getKeyChar()) {
                case 's':
                case 'S':
                    mode = 1;
                    break;
                case 'u':
                case 'U':
                    mode = 3;
                    break;
                case 'r':
                case 'R':
                    mode = 4;
                    break;
                case 'c':
                case 'C':
                    messageText.selectAll();
                    messageText.copy();
                    break;
                case 'm':
                case 'M':
                    mode = 5;
                    break;
                case 't':
                case 'T':
                    mode = 6;
                    break;
                }
                break;
            case 1:
                key1 = e.getKeyChar();
                mode = 2;
                break;
            case 2:
                key2 = e.getKeyChar();
                mode = 0;
                cryptogram.set(key1, key2);
                break;
            case 3:
                key1 = e.getKeyChar();
                cryptogram.unset(key1);
                mode = 0;
                break;
            case 4:
                key1 = e.getKeyChar();
                if ((key1 == 'y') || (key1 == 'Y'))
                    cryptogram.unsetAll();
                mode = 0;
                break;
            case 5:
                switch(e.getKeyChar()) {
                case 'e':
                case 'E':
                    cryptogram.setMessage(EntryDialog.getEntry());
                    mode = 0;
                    break;
                case 'Q':
                case 'q':
                default:
                    mode = 0;
                }
                break;
            case 6:
                switch(e.getKeyChar()) {
                case 'l':
                case 'L':
                    cryptogram.letterDistribution(letters, counts);
                    for (int i=0; i<26; i++)
                        tempStr.append("" + letters[i] + " " + counts[i] + "\n");
                    EntryDialog.showText("Letter Distribution", tempStr.toString());
                    mode = 0;
                    break;
                case 'w':
                case 'W':
                    Cryptogram.Pair[] words = cryptogram.wordDistribution();
                    for (int i=0; i<words.length; i++)
                        tempStr.append(pad(words[i].word, 10) + " " + words[i].count + "\n");
                    EntryDialog.showText("Word Distribution", tempStr.toString());
                    mode = 0;
                    break;
                case 'r':
                case 'R':
                    cryptogram.randomizeKeys();
                    mode = 0;
                    break;
                case 't':
                case 'T':
                    EntryDialog.showText("ROT Results", cryptogram.getRot());
                    mode = 0;
                    break;
                case 'Q':
                case 'q':
                default:
                    mode = 0;
                }
                break;
            }
            refresh();
        }
    }
}
